# import unittest


# class ExceptionsTestCase(unittest.TestCase):
# from manatus import exceptions

# def test_something(self):
#     self.assertEqual(True, False)


# if __name__ == '__main__':
#     unittest.main()
